#include "component_enemy.h"

/* DEPENDENCIES: Character */

/* Sets enemy component info, with an atlas. */
void
component_enemy_init
(
	ComponentEnemy* self,
	ECS* ecs,
	Texture texture,
	ivec2 frameSize,
	ivec2 atlasSize,
	vec2 size,
	vec2 physicsSize,
	vec3 position,
	f32 friction,
	f32 velocityMax,
	u32 healthValue,
	bool isBounce,
	bool isAffectedByGravity,
	bool isSolid,
	u32 goodieCount
)
{
	ComponentCharacter* character;
	ComponentActionOnDeleteDropGoodies* actionOnDeleteDropGoodies;
	
	character = ecs_component_get(ecs, ECS_COMPONENT_CHARACTER, self->component.id);
	actionOnDeleteDropGoodies = ecs_component_get(ecs, ECS_COMPONENT_ACTION_ON_DELETE_DROP_GOODIES, self->component.id);
	
	component_character_init
	(
		character,
		ecs,
		texture,
		frameSize,
		atlasSize,
		size,
		physicsSize,
		position,
		friction,
		velocityMax,
		COMPONENT_ENEMY_ALLEGIANCE,
		healthValue,
		isBounce,
		isAffectedByGravity,
		isSolid
	);

	component_action_on_delete_drop_goodies_init(actionOnDeleteDropGoodies, goodieCount);
}

/* Runs on character add. */
void
component_enemy_add(ComponentEnemy* self, ECS* ecs)
{
	ecs_component_add(ecs, ECS_COMPONENT_CHARACTER, self->component.id);
	ecs_component_add(ecs, ECS_COMPONENT_ACTION_ON_DELETE_DROP_GOODIES, self->component.id);
}

/* Runs on enemy deletion. */
void
component_enemy_delete(ComponentEnemy* self, ECS* ecs)
{
	ecs_component_delete(ecs, ECS_COMPONENT_CHARACTER, self->component.id);
	ecs_component_delete(ecs, ECS_COMPONENT_ACTION_ON_DELETE_DROP_GOODIES, self->component.id);
}
